package cache

import (
	"dc_golang_server_1/dbmodel"
	"dc_golang_server_1/logger"
	"fmt"
	"go.uber.org/zap"
	"log"
	"time"
)

func initAreaMap() {
	fmt.Println(time.Now(), "cache: initDeviceMap begin")
	var areaID int32
	var temp AreaStruct
	rows, err := dbmodel.PgDb.Query("SELECT id,name,status FROM b_area")
	defer rows.Close()
	if err != nil {
		log.Panicln("SelectAreaToMap rows", err)
	}
	for rows.Next() {
		err = rows.Scan(&areaID, &temp.Name, &temp.Status)
		if err != nil {
			log.Panicln("SelectAreaToMap rows.Scan", err)
		}
		phone20DeviceMap.Store(areaID, temp)
	}
	// 处理完毕后，需要判断一次遍历过程中是否有错误产生
	if err = rows.Err(); err != nil {
		logger.Log.Error("rows.Err()",
			zap.Error(err),
			zap.String("src", "initAreaMap"))
		fmt.Println(time.Now(), "initAreaMap rows.Err()", err)
	} else {
		fmt.Println(time.Now(), "cache: initAreaMap end")
	}
}

func GetAreaInfo(areaID int32) AreaStruct {
	if info, ok := areaMap.Load(areaID); ok { //map有
		if v, ok2 := info.(AreaStruct); ok2 { //类型断言正确
			return v //,true
		} else { //类型断言失败
			areaMap.Delete(areaID)
			logger.Log.Error("AreaMap 类型断言错误",
				zap.Int32("areaID", areaID),
				zap.String("src", "GetAreaInfo"))
		}
	}
	var temp AreaStruct
	var err error
	for i := 0; i < 100; i++ {
		err = dbmodel.PgDb.QueryRow("SELECT name,status FROM b_area WHERE id=$1",
			areaID).Scan(&temp.Name, &temp.Status)
		if err != nil {
			fmt.Println("GetAreaInfo  SELECT:", i, err)
			time.Sleep(3 * time.Millisecond)
			continue
		}
		areaMap.Store(areaID, temp)
		logger.Log.Error("数据库有而缓存没有 GetAreaInfo",
			zap.Int32("areaID", areaID),
			zap.Error(err),
			zap.String("src", "GetAreaInfo"))
		return temp //,true
	}
	logger.Log.Error("GetAreaInfo SELECT operator_id,status FROM b_area WHERE id=$1",
		zap.Int32("areaID", areaID),
		zap.Error(err),
		zap.String("src", "getDeviceInfo"))
	return temp //,false
}
