package main

import (
	"dc_golang_server_1/api"
	"dc_golang_server_1/cache"
	"dc_golang_server_1/config"
	"dc_golang_server_1/dbmodel"
	"dc_golang_server_1/devproduct/dcphone20"
	"dc_golang_server_1/logger"
	"os"
	"runtime"
	"syscall"
)

//windows
var (
	kernel32         = syscall.MustLoadDLL("kernel32.dll")
	procSetStdHandle = kernel32.MustFindProc("SetStdHandle")
)

func setStdHandle(stdhandle int32, handle syscall.Handle) error {
	r0, _, e1 := syscall.Syscall(procSetStdHandle.Addr(), 2, uintptr(stdhandle), uintptr(handle), 0)
	if r0 == 0 {
		if e1 != 0 {
			return error(e1)
		}
		return syscall.EINVAL
	}
	return nil
}

// redirectStderr to the file passed in
func redirectStderrWindows(f *os.File) {
	err := setStdHandle(syscall.STD_ERROR_HANDLE, syscall.Handle(f.Fd()))
	if err != nil {
		logger.Log.Panic(err.Error())
	}
	// SetStdHandle does not affect prior references to stderr
	os.Stderr = f
}

//windows end

//func redirectStderrLinux(f *os.File) {
//	err := syscall.Dup2(int(f.Fd()), int(os.Stderr.Fd()))
//	if err != nil {
//		logger.Log.Panic(err.Error())
//	}
//}

//func init() {
//
//}

func initConfigAndLogger() {
	config.ConfigInit()
	logger.InitLogger()

	f, err := os.OpenFile(config.LogPath+"panic.log", os.O_CREATE|os.O_APPEND|os.O_WRONLY, 0755)
	if err != nil {
		logger.Log.Panic(err.Error())
	}

	redirectStderrWindows(f)
	//redirectStderrLinux(f)
}

func main() { //
	runtime.GOMAXPROCS(runtime.NumCPU())

	initConfigAndLogger()

	dbmodel.InitDb()
	defer dbmodel.CloseDb()

	cache.Init()

	dcphone20.NewDCPhone20()
	//dcdrinking60.NewDCDrinking60()

	api.InitRouter()
}
